/***************************************************************************
 *   Copyright (C) 2004, 2006 Symbian Software Ltd.                        *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 ***************************************************************************/

#include "diagnostic.h"
#include "evaluator.h"
#include "categorical.h"
#include "line_despatch.h"
#include "if_control.h"
#include "args.h"
#include "symbol_table.h"
#include "io.h"
#include "chew.h"
#include "line_edit.h"
#include <stdio.h>


/*! \ingroup main_module
 * \file main.c
 *	This file implments the Sunifdef program
 */

/*! \addtogroup main_module */
/*@{*/

/*! Perform all module initialisations */
static void		ready(void);

/*! Execute a sunifdef process */
static void		process(void);

/*! The main program. */
int
main(int argc, char *argv[])
{
	ready();
	atexit(exit_diagnostics);
	parse_executable(argv);
	parse_args(argc,argv);
	finish_args();
	process();
	give_up_confused(); /* bug */
	return 0;
}

static void
line_debug(int line)
{
	if (line < 0) {
		/* Call with negative value to write line numbers */
		printf("%d\n",(int)GET_PUBLIC(io,line_num));
	}
	else if (line > 0 && (size_t)line == GET_PUBLIC(io,line_num)) {
		/* Call with positive line number and put breakpoint here ... */
		fputs("Here",stderr);	
	}
}

static void
ready(void)
{
	INITIALISE(chew);
	INITIALISE(diagnostic);
	INITIALISE(evaluator);
	INITIALISE(if_control);
	INITIALISE(io);
	INITIALISE(line_edit);
	INITIALISE(args);
	INITIALISE(symbol_table);
	INITIALISE(line_despatch);
	INITIALISE(categorical);
}
	
static void
process(void)
{
	while(open_io()) {
		line_type_t lineval;
		for (;!input_eof();) {
			line_debug(0);
			lineval = eval_line();
			if (!weed_categorical_directive(lineval)) {
				transition(lineval);
			}
			debug(DBG_19,lineval);
		}
	}
	exit(exitcode());
}
/*@}*/

/* EOF */
